#include <iostream>
#include <vector>
#include <cmath>
#include "module.h"
using namespace std;

bool asked[2049][2049];
int cache[2049][2049];

int _ask(int x,int y)
{
	if (asked[x][y])
	{
		return cache[x][y];
	} else
	{
		asked[x][y] = true;
		return cache[x][y]=ask(x,y);
	}
	
}


/************************************/
struct Solve {
	vector<int> x, y;
	int n;
	int s;
	int sz;

	bool operator()(int N,int SZ) {
		n = N;
		sz = SZ;
		return find(1, 1,sz, sz);
	}

	bool find(int x1, int y1, int x2, int y2) {
		int r;
		int xr, yr;
		int xr1, yr1;
		int xr2, yr2;

		if (x1 < 1 || y1 < 1 || x2 > sz || y2 > sz)
			return false;

		build(x1, y1, x2, y2);
		r = search();
		if (r < 0 || r >= s)
			return false;
		xr = x[r];
		yr = y[r];
		if (ask(xr, yr) == n)
		{
			answer(x[r],y[r]);
		}

		//---------------
		build(x1, yr, xr, yr);
		r = search();
		if (r < 0 || r > s)
			return false;
		xr1 = x[r];
		if (ask(x[r], y[r]) == n)
		{
			answer(x[r],y[r]);
		}


		build(x1, yr, x1, y2);
		r = search();
		if (r < 0 || r > s)
			return false;
		yr1 = y[r];
		if (ask(x[r], y[r]) == n)
		{
			answer(x[r],y[r]);
		}


		if (find(x1, yr, xr1 - 1, yr1 - 1))
			return true;

		//----------------
		build(xr, y1, x2, y1);
		r = search();
		if (r < 0 || r > s)
			return false;
		xr2 = x[r];
		if (ask(x[r], y[r]) == n)
		{
			answer(x[r],y[r]);
		}


		build(xr, y1, xr, yr);
		r = search();
		if (r < 0 || r > s)
			return false;
		yr2 = y[r];
		if (ask(x[r], y[r]) == n)
		{
			answer(x[r],y[r]);
		}


		return find(xr, y1, xr2 - 1, yr2 - 1);
	}

	void build(int x1, int y1, int x2, int y2) {
		double t;
		int i;

		x.clear();
		y.clear();
		if (x2 - x1 > y2 - y1) {
			s = x2 - x1 + 1;
			for (i = 0; i < s; i++) {
				t = ceil((double)i/s*(y2 - y1));
				x.push_back(x1 + i);
				y.push_back(y1 + (int)t);
			}
		}
		else {
			s = y2 - y1 + 1;
			for (i = 0; i < s; i++) {
				t = ceil((double)i/s*(x2 - x1));
				x.push_back(x1 + (int)t);
				y.push_back(y1 + i);
			}
		}
		x.push_back(x2 + 1);
		y.push_back(y2 + 1);
	}

	private:
	
	int search() {
		return bin_search();
	}

	//incremental search
	int inc_search() {
		int i;

		for (i = 0; i < s; i++)
			if (ask(x[i], y[i]) >= n)
				break;
		return i;
	}

	//binary search
	int bin_search() {
		int l = -1, r = s;
		int m, t;

		while (l + 1 < r) {
			m = (l + r)/2;
			t = ask(x[m], y[m]);
			if (t > n) r = m;
			if (t < n) l = m;
			if (t == n) return m;
		}
		return r;
	}

	//interpolation search
	int int_search() {
		int l = 0, r = s - 1;
		int vl, vr;
		int m, vm;
		double t;

		vr = ask(x[r], y[r]);
		if (vr < n) return s;
		if (vr == n) return r;
		vl = ask(x[l], y[l]);
		if (vl >= n) return l;
		while (l + 1 < r) {
			t = ((double)(n - vl)/(double)(vr - vl))*(r - l);
			m = (int)floor(t) + l;
			if (m == l)
				m = l + 1;
			if (m == r)
				m = r - 1;

			vm = ask(x[m], y[m]);
			if (vm > n) {
				r = m;
				vr = vm;
			}
			if (vm < n) {
				l = m;
				vl = vm;
			}
			if (vm == n) return m;
		}
		return r;
	}

	int ask(int x, int y) {
		static int calls = 0;

		if (x < 1 || x > sz || y < 1 || y > sz)
			return n - 1;
		return ::_ask(x,y);
	}
};

//////////////////////////////////////////////////////////////


void solve(int aLeft, int aRight,int aTop,int aBottom, int aValue)
{
	
	if (aLeft>aRight || aTop>aBottom)
	{
		return;
	}


	if ((aLeft==aRight || aLeft+1==aRight) && (aTop==aBottom || aTop+1==aBottom))
	{
		for (int i=aTop;i<=aBottom;i++)
		{
			for (int j=aLeft;j<=aRight;j++)
			{
				bool result = _ask(i,j)==aValue;
				if (result)
				{
					answer(i,j);
				}
			}
		}
		return;
	}

	int v = _ask(aTop,aLeft);

	if (v>aValue)
	{
		return;
	}
	if (v==aValue)
	{
		answer(aTop,aLeft);
	}

	v = _ask(aBottom,aRight);

	if (v<aValue)
	{
		return;
	}

	if (v==aValue)
	{
		answer(aBottom,aRight);
	}

	int mx = (aLeft+aRight)/2;
	int my = (aTop+aBottom)/2;

	v = _ask(my,mx);

	if (v==aValue)
	{
		answer(my,mx);
	}

	if (v<aValue)
	{
		solve(mx,aRight,my,aBottom,aValue);
		solve(mx+1,aRight,aTop,my-1,aValue);		
		solve(aLeft,mx-1,my,aBottom,aValue);
	}
	else
	{
		solve(aLeft,mx,aTop,my,aValue);
		solve(mx+1,aRight,aTop,my,aValue);
		solve(aLeft,mx,my+1,aBottom,aValue);
	}	
}



void solveBest(int maxAge,int minWeight,int goal) {
	int i, j,t;

	for (i=maxAge, j=minWeight; ; ) {
		if ((t=_ask(i, j))==goal) {
			answer(i,j);
		} else if (t<goal) j++;
		else i--;
	}
}

void solveVelin(int N, int goal) {
	int i, j;

	int rems=0;
	int rem[1024*1024][2];
	for (i=1; i<=N; i++) for (j=1; j<=N; j++) {
		rem[rems][0] = i;
		rem[rems][1] = j;
		rems++;
	}

	while (rems) {
		int bestb=rems+1, bestl=rems+1, bestw=rems;
		for (int k=0; k<rems; k++) {
			int a=0, b=0;
			int bi, sm;
			for (int l=0; l<rems; l++) {
				if (rem[l][0]<rem[k][0] || rem[l][1]<rem[k][1]) a++;
				if (rem[l][0]>rem[k][0] || rem[l][1]>rem[k][1]) b++;
			}
			bi = max(a,b);
			sm = a+b-bi;
			if (bi<bestb || (bi==bestb && sm<bestl)) {
				bestb = bi;
				bestl = sm;
				bestw = k;
			}
		}
		int a=rem[bestw][0], b=rem[bestw][1];
		int val = ask(a, b);
		if (val==goal) {
			answer(a,b);
			return;
		}
		if (val>goal) {
			for (int k=0; k<rems; k++) {
				if (rem[k][0]>=a && rem[k][1]>=b) {
					rems--;
					rem[k][0] = rem[rems][0];
					rem[k][1] = rem[rems][1];
					k--;
				}
			}
		} else {
			for (int k=0; k<rems; k++) {
				if (rem[k][0]<=a && rem[k][1]<=b) {
					rems--;
					rem[k][0] = rem[rems][0];
					rem[k][1] = rem[rems][1];
					k--;
				}
			}
		}
	}
}

void b_search_r(int N,int K,int aRow) {
	int l = -1, r = N;
	int m, t;

	while (l + 1 < r) {
		m = (l + r)/2;
		t = ask(aRow,m);
		if (t > K) r = m;
		if (t < K) l = m;
		if (t == K) {
			answer(aRow,m);
		}
	}
	if (ask(aRow,r)==K)
	{
		answer(aRow,r);
	}
}

void b_search_c(int N,int K,int aCol) {
	int l = -1, r = N;
	int m, t;

	while (l + 1 < r) {
		m = (l + r)/2;
		t = ask(m,aCol);
		if (t > K) r = m;
		if (t < K) l = m;
		if (t == K) {
			answer(m,aCol);
		}
	}
	if (ask(r,aCol)==K)
	{
		answer(r,aCol);
	}
}

void solve_bin(int N,int K)
{

	for (int i=N;i>=1;i--)
	{
		int start = ask(1,i);
		if (K<=start)
		{
			if (K==start)
			{
				answer(1,i);
			}
			continue;
		}
		int final = ask(N,i);
		if (K>=final)
		{
			if (K==final)
			{
				answer(N,i);
			}
			continue;
		}
		b_search_c(N,K,i);

	}
}

int main () 
{
	long N;
	long K;
	init(&N,&K);
	//solveBest(N,1,K);
	Solve s;
	s(K,N);
	//solve(1,N,1,N,K);
	//solve_bin(N,K);
	//solveVelin(N,K);
	return 0;
}
